<?php

namespace LakeDrops\Drupal8ScaffoldDeveloper;

use LakeDrops\Component\Composer\BaseHandler;
use LakeDrops\Component\Dotenv\Dotenv;
use LakeDrops\Docker4Drupal\Handler as D4D;
use Symfony\Component\Filesystem\Filesystem;

/**
 * Class Handler.
 *
 * @package LakeDrops\Drupal8ScaffoldDeveloper
 */
class Handler extends BaseHandler {

  /**
   * Post project create event to execute the scaffolding.
   *
   * @throws \Twig_Error_Loader
   * @throws \Twig_Error_Runtime
   * @throws \Twig_Error_Syntax
   * @throws \Exception
   */
  public function setupLakeDropsProject() {

    // We only do the fancy stuff for developers.
    if (!$this->isDevMode()) {
      return;
    }

    $options = $this->getOptions();
    $fs = new Filesystem();
    $installationManager = $this->composer->getInstallationManager();

    $drupalCorePackage = $this->getDrupalCorePackage();
    /** @noinspection NullPointerExceptionInspection */
    $corePath = $installationManager->getInstallPath($drupalCorePackage);

    // Directory where the root project is being created.
    $projectRoot = getcwd();
    // Directory where Drupal's index.php is located.
    $webRoot = dirname($corePath);
    // Directory where this plugin is being installed.
    $pluginRoot = $installationManager->getInstallPath($this->getPackage('lakedrops/d8-project-scaffold-developer'));

    // Create contrib/custom dirs for modules, profiles and themes.
    foreach ([
      $webRoot . '/modules',
      $webRoot . '/profiles',
      $webRoot . '/themes',
    ] as $dir) {
      foreach (['contrib', 'custom'] as $subdir) {
        $path = $dir . '/' . $subdir;
        if (!$fs->exists($path)) {
          $fs->mkdir($path, 0775);
        }
        if (!$fs->exists($path . '/.gitkeep')) {
          $fs->touch($path . '/.gitkeep');
        }
        if ($options['chg-acl']) {
          $fs->chgrp($path, $options['webserver-groupname'], TRUE);
        }
      }
    }

    // Create and protect config, settings and files directories.
    foreach ([
      'config/default/sync',
      'settings/default',
      'files/default/files',
      'files/default/files/css',
      'files/default/files/js',
      'files/default/files/languages',
      'files/default/files/translations',
      'files/default/private',
      'drush',
      'tests',
    ] as $dir) {
      $path = $projectRoot . '/' . $dir;
      if (!$fs->exists($path)) {
        $fs->mkdir($path);
        $fs->copy($pluginRoot . '/templates/.htaccess', $path . '/.htaccess');
      }
      if ($options['chg-acl']) {
        $fs->chgrp($path, $options['webserver-groupname'], TRUE);
      }
    }

    // Link Drupal site's files and private directory.
    foreach (['files', 'private'] as $dir) {
      if (!$fs->exists($webRoot . '/sites/default/' . $dir)) {
        $rel = substr($fs->makePathRelative($projectRoot . '/files/default/' . $dir, $projectRoot . '/' . $webRoot . '/sites/default/' . $dir), 3, -1);
        $fs->symlink($rel, $webRoot . '/sites/default/' . $dir);
        $fs->chmod($webRoot . '/sites/default/' . $dir, 0775);
      }
    }

    // Link Drupal site's settings files.
    $options['hash'] = str_replace(['+', '/', '='], ['-', '_', ''], base64_encode(random_bytes(55)));
    $twig_loader = new \Twig_Loader_Array([]);
    $twig = new \Twig_Environment($twig_loader);

    $settings_files = [
      'settings.php',
      'services.yml',
    ];
    if (!$this->isCiContext()) {
      $settings_files[] = 'settings.local.php';
      $settings_files[] = 'development.services.yml';
    }
    foreach ($settings_files as $template) {
      $file = $projectRoot . '/settings/default/' . $template;
      $link = $webRoot . '/sites/default/' . $template;
      if (!$fs->exists($link)) {

        $twig_loader->setTemplate($template, file_get_contents($pluginRoot . '/templates/' . $template));
        $rendered = $twig->render($template, $options);
        file_put_contents($file, $rendered);

        $rel = substr($fs->makePathRelative($file, $projectRoot . '/' . $link), 3, -1);
        $fs->symlink($rel, $link);
      }
      $fs->chmod($file, 0664);
      if ($options['chg-acl']) {
        $fs->chgrp($file, $options['webserver-groupname']);
      }
    }

    // Copy Drush policy and readme.
    foreach (['policy.drush.inc', 'README.md'] as $template) {
      $fs->copy($pluginRoot . '/templates/drush/' . $template, 'drush/' . $template);
    }

    // Copy Test files.
    foreach (['phpcs.files', 'phpunit.xml.dist'] as $template) {
      $fs->copy($pluginRoot . '/templates/tests/' . $template, 'tests/' . $template);
    }

    // Link Drupal site's config directory.
    if (!$fs->exists($webRoot . '/sites/default/files/config')) {
      $rel = substr($fs->makePathRelative($projectRoot . '/config/default', $projectRoot . '/files/default/files/config'), 3, -1);
      $fs->symlink($rel, $webRoot . '/sites/default/files/config');
    }

    // Tasks we only do the first time this is run.
    if (!$fs->exists($projectRoot . '/.git')) {

      // Remove template files.
      foreach (['.gitlab-ci.yml', 'README.md'] as $file) {
        $fs->remove($file);
      }

      // Initialize local git working copy.
      try {
        $this->git('init');
        $this->git('add *');
        $this->git('commit -m "Initial commit"');
      }
      catch (\Exception $ex) {
        // We're ignoring this for now.
      }
    }

    // If available, also configure Docker4Drupal.
    if ($this->getPackage('lakedrops/docker4drupal')) {
      $handler = new D4D($this->composer, $this->io);
      $handler->configureProject();
    }
  }

  /**
   * Retrieve configuration for this package.
   *
   * @return array
   *   The settings from the extra configuration.
   */
  protected function getOptions(): array {
    $extra = $this->composer->getPackage()->getExtra() + ['d8-project-scaffold' => []];
    $options = $extra['d8-project-scaffold'] + [
      'chg-acl' => FALSE,
      'webserver-username' => 'www-data',
      'webserver-groupname' => 'www-data',
      'filter_protocols' => [
        'http',
        'https',
        'ftp',
        'news',
        'nntp',
        'tel',
        'telnet',
        'mailto',
        'irc',
        'ssh',
        'sftp',
        'webcal',
        'rtsp',
      ],
      'cors_config' => [
        'enabled' => 'false',
        'allowedOrigins' => '*',
      ],
    ];
    $env = new Dotenv('lakedrops', $this->io);
    return $env->replaceEnvironmentVariables($options);
  }

}
